#!/bin/sh

. /etc/init.d/rc.functions
. /etc/conf.d/rc.conf

write_peer6()
{
    for index in `seq ${ip6}` ; do
        eval ip6addr="\$ip6addrs$index"
        if [ -n "$ip6addr" ] ; then
            if [ "${thisnode}" == "${host}" ] ; then
                echo "$ip6addr $host.$domain $host" >> $hostsfile
            else
                if [ "$peer" == "no" ] ; then
                    peer="yes"
                    echo "$ip6addr $host.$domain $host" >> $hostsfile
                    echo "$host" >> $cfpeersfile
                else
                    echo "$ip6addr $host.$domain $host" >> $hostsfile.nopeers
                fi
            fi
        fi
    done
}

write_peer4()
{
    for index in `seq ${ip}` ; do
        eval ipaddr="\$ipaddrs$index"
        if [ -n "$ipaddr" ] ; then
            if [ "${thisnode}" == "${host}" ] ; then
                echo "$ipaddr $host.$domain $host" >> $hostsfile
            else
            	if [ "$peer" == "no" ] ; then
                    peer="yes"
                    echo "$ipaddr $host.$domain $host" >> $hostsfile
                    echo "$host" >> $cfpeersfile
                else
                    echo "$ipaddr $host.$domain $host" >> $hostsfile.nopeers
                fi
            fi
        fi
    done
}

calc_peers()
{
    if [ ${GLOBAL_CFHOPS} -gt 4 ] || [ ${GLOBAL_CFHOPS} -lt 1 ] ; then
        GLOBAL_CFHOPS=1
    fi
    let GLOBAL_CFHOPS++
    STANDARD_IFS=$IFS
    gen_default_hosts ${hostsfile}
    echo localhost > ${cfpeersfile}
    echo "# unreachable peers" > ${hostsfile}.nopeers
    echo "# reachable peers and sorted local entries" >> ${hostsfile}
    for host in ${networkdir}/*.conf ; do
        host=${host##*/}
        host=${host%.conf}
        ip=0
        ip6=0
        IFS="
"
        for line in `cat ${hostsfile}.tmp | grep -e ${host}.${domain} | grep -e "^[0-9,a-f]"` ; do
            cand=`echo ${line} | awk '{ print $1 ; }'`
            if [ "${thisnode}" == "${host}" ] ; then
                if echo $cand | grep  \\. >/dev/null ; then
                    let ip++
                    eval ipaddrs$ip=$cand
                elif echo $cand | grep \\: >/dev/null ; then
                    let ip6++
                    eval ip6addrs$ip6=$cand
                fi
            else
                if echo $cand | grep \\. >/dev/null ; then
                    if traceroute -q 2 -w 2 -m ${GLOBAL_CFHOPS} $cand | tail -n 1 2> /dev/null | grep $cand 1> /dev/null ; then
                        let ip++
                        eval ipaddrs$ip=$cand
                    else
                        echo "${line}" >> ${hostsfile}.nopeers
                    fi
                elif echo $cand | grep \\: >/dev/null ; then
                    if traceroute6 -q 2 -w 2 -m ${GLOBAL_CFHOPS} $cand 2> /dev/null | grep $cand 1> /dev/null ; then
                        let ip6++
                        eval ip6addrs$ip6=$cand
                    else
                        echo "${line}" >> ${hostsfile}.nopeers
                    fi
                fi
            fi
        done
        IFS=${STANDARD_IFS}
        peer="no"
        if [ "${GLOBAL_CFNET6}"  == "yes" ] ; then
            write_peer6
            write_peer4
        else
            write_peer4
            write_peer6
        fi
    done
    cat ${hostsfile}.nopeers >> ${hostsfile}
    rm ${hostsfile}.nopeers
}        

domain=`cat /etc/conf.d/domainname`
thisnode=`cat /etc/conf.d/hostname`
networkdir="/etc/network.d"
hostsfile="/etc/hosts"
cfpeersfile="/var/cfengine/inputs/cfpeers"

case $1 in
     read)
         logger "$0 $@"         
         if [ -f ${cfpeersfile} ] ; then
             echo `cat ${cfpeersfile}`
         else 
             echo localhost
         fi
         ;;
     calc)
         logger "$0 $@"
         cp ${hostsfile} ${hostsfile}.tmp
         calc_peers
         rm ${hostsfile}.tmp
         chmod 644 ${hostsfile}
         chmod 644 ${cfpeersfile}
         echo `cat ${cfpeersfile}`
         ;;
     *)
         echo "Usage $0 calc|read"
         exit 1
         ;;
esac

